package givenwhenthen;

/**
 * Create an anonymous subclass of this class for each 'given' instance.
 * 
 * @author Kerry Buckley
 */
public abstract class Given {
    private String desc;

    public Given(final String desc) {
        this.desc = desc;
    }

    /**
     * This method must be implemented to perform the behaviour corresponding to
     * the particular 'given' instance.
     */
    public abstract void execute();

    /**
     * Runs a 'given' instance. Intended to be statically imported so the test
     * can use it as follows:
     * 
     * <pre>
     * given(aGivenInstance).when(aWhenInstance).then(&quot;something should happen&quot;);
     * </pre>
     * 
     * @param given
     *            the instance to run
     * @return the same instance which was passed in
     */
    public static Given given(final Given given) {
        given.report();
        given.execute();
        return given;
    }

    /**
     * Runs a 'when' instance. Intended to be statically imported so the test
     * can use it as follows:
     * 
     * <pre>
     * given(aGivenInstance).when(aWhenInstance).then(&quot;something should happen&quot;);
     * </pre>
     * 
     * This is here rather than in {@link When} to allow the chaining described
     * above. Defers throwing exceptions to allow the complete given-when-then
     * to be reported in the case where an exception is expected (@see
     * {@link When#then(String)}).
     * 
     * @param when
     *            the instance to run
     * @return the same instance which was passed in
     */
    public When when(final When when) {
        try {
            when.execute();
        } catch (RuntimeException t) {
            when.setThrown(t);
        }
        return when;
    }

    /**
     * Does not participate functionally in the test, but allows the 'then'
     * description to be reported. Exists in this class as well as in
     * {@link When} to allow for cases where no 'when' is required.
     * 
     * @param then
     *            a descriptive string
     */
    public void then(String then) {
        When.reportThen(then);
    }

    private void report() {
        System.out.println("\nGiven: " + desc);

    }
}
